/*
 * File: Main.c
 * 
 * Processor: PIC32MX150(170)F256B w/Ext OSC @ 4MHz, PLL to 40 MHz, 3.3v
 * Program: 23_TTS
 * Compiler: XC32 (v2.10, MPLAX v5.05)
 * Program Version: 1.0
 *                             
 * Program Description: This program allows PIC32MX150/170 (with 28 pins) to use  
 *                      the PWM module to perform TTS.
 * 
 * Hardware Description: A external amplifier is connected to pin7 and a USB-UART 
 *                       bridge is connected to pin11 (SAM Tx) and pin12 (SAM Rx).
 *                       A BUSY pin is available on pin17.
 * Change History:
 *
 *
 * Many thanks to:
 * 
 * - Sebastian Macke (www.simulationcorner.net - he made SAM 
 * on microcontrollers possible in the first place!)
 *  
 * - http://www.toughdev.com/content/2014/09/english-text-to-speech-on-a-pic-microcontroller/
 * he ported Sebastian's code to the PIC24.
 * 
 * - Armstrong Subero - he ported the PIC24 code to PIC32
 * 
 * and special thanks to:
 *  
 * - Peter Mather - without him there would be no SAM  
 * for the 28-pin chip in such good quality!!!
 * 
 * This code was modified by Frank N. Furter for easier use...
 *                      
 *  November 2018
 */


/*******************************************************************************
 * Includes and Defines
 ******************************************************************************/
#include "PIC32MX270_STD.h"
#include "delay.h"
#include "uart1.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "sam.h"
#include "reciter.h"
#include "tts.h"
#define BUSY LATBbits.LATB8 //Change to whatever pin you want
extern unsigned char speed;
extern unsigned char pitch;
extern unsigned char mouth;
extern unsigned char throat;
extern unsigned char singpitch;
extern int debug;
/*******************************************************************************
* Function Name: void initMain(void)
*
* Specification: initializes main
*
*******************************************************************************/
void initMain(void)
{
    // unlock PPS
    systemReg_Unlock();
    CFGCONbits.IOLOCK = 0;
    TRISBbits.TRISB8 = 0;  // set PINB8 as output, Change to whatever pin you want
    BUSY = 1; //set Busy while we get started

    RPB3Rbits.RPB3R = 0x0005;   //RB3->OC1
    RPB4Rbits.RPB4R = 0x0001;   //RB4->UART1 TX
    U1RXR = 0b0010;     //RA4->UART1 RX
    // lock PPS
    CFGCONbits.IOLOCK = 1;
    systemReg_Lock();   
}

int debug = 0;

int main()
{	
    char input[512];
    char tempstr[2];
    int j=12345;
    int i=12345;
    int phonetic = 0;
    int sing = 0;

    
    uc_init();
    initMain();
    
    // initialize UART
    UART1_Initialize(); 
    delay_ms(300);
    
    // initialize the TTS module
    initTTS();
      __XC_UART = 1;
    ShowInstructionsInfo ();
            
	while(1)
	{
        int speak=1;
        // configure printf to use UART1
        // begin speaking
        printf(">");
        BUSY = 0; //ready to receive input


        input[0]=UART1_Read();
        if(input[0]=='$'){
            
            for(j=0; j<256; j++) input[j] = 0;
            j=0;
        
                while((input[j++]=UART1_Read())!=13 && j<254);
                
                //sing = 0;
                speak = 0;
                
                for(j=0; input[j] != 0; j++)
                    input[j] = toupper((int)input[j]);
 
                if (strcmp(input, "HELP\r")==0)
                {
                    ShowInstructionsInfo ();
                } else

                if (strcmp(input, "TABLE\r")==0)
                {
                    ShowTABLE ();
                } else
 
                if (strcmp(input, "DEFAULT\r")==0)
                {//reset all values to default settings
                    printf("All values are set to default!\r\n");
                    speed = 72;
                    pitch = 64;
                    mouth = 128;
                    throat = 128;
                    phonetic = 0;
                    sing = 0;
                    debug = 0;        
                } else

                if (strcmp(input, "STATUS\r")==0)
                {
                    printf("\r\n");
                    printf("\r\n*** STATUS ***\r\n");
                    //printf("\r\n");
                    printf("SPEED  = %d \r\n",speed);
                    printf("PITCH  = %d \r\n",pitch);
                    printf("MOUTH  = %d \r\n",mouth);
                    printf("THROAT = %d \r\n",throat);
                    
                    //printf("phonetic  = %d \r\n",phonetic);
                    //printf("sing  = %d \r\n",sing);
                    
                    printf("MODE   = ");
                    if (!phonetic && !sing){     //Textmode
                        printf("Text mode\r\n");
                    }
                    
                    if (phonetic && !sing){     //Phonetic mode
                        printf("Phonetic mode\r\n");
                    }
                    
                    if (!phonetic && sing){     //Sing mode
                        printf("Sing mode\r\n");
                    }
                    
                    if (debug){
                        printf("DEBUG IS ON\r\n");    
                    }else
                        printf("DEBUG IS OFF\r\n");
                    
                
                    printf("\r\n");
                } else                    
                    
                if (strncmp(input, "SPEED=", 6)==0)
                {
                    speed = atoi(strncpy(tempstr, input+6, 3));
                    printf("\r\nNew SPEED %d \r\n",speed);
                } else

                if (strncmp(input, "PITCH=", 6)==0)
                {
                    pitch = atoi(strncpy(tempstr, input+6, 3));
                    printf("\r\nNew PITCH %d \r\n",pitch);
                } else
                
                if (strncmp(input, "MOUTH=", 6)==0)
                {
                    mouth = atoi(strncpy(tempstr, input+6, 3));
                    printf("\r\nNew MOUTH %d \r\n",mouth);
                } else
                    
                if (strncmp(input, "THROAT=", 7)==0)
                {
                    throat = atoi(strncpy(tempstr, input+7, 3));
                    printf("\r\nNew THROAT %d \r\n",throat);
                } else
                    
                if (strcmp(input, "TEXT\r")==0)
                {
                    phonetic = 0;
                    sing = 0;
                    printf("Text Mode active!\r\n");
                } else
                    
                if (strcmp(input, "PHONEME\r")==0)
                {
                    phonetic = 1;
                    sing = 0;
                    printf("Phonetic Mode active!\r\n");
                } else

                if (strcmp(input, "SING\r")==0)
                {
                    phonetic = 0;
                    sing = 1;
                    printf("Sing Mode active!\r\n");
                } else                

                if (strcmp(input, "DEMO1\r")==0)
                {
                    DemoSpeaking1();
                } else

                if (strcmp(input, "DEMO2\r")==0)
                {
                    DemoSpeaking2();
                } else
                    
                if (strcmp(input, "DEMO3\r")==0)
                {
                    DemoSinging();
                } else
                    
                if (strcmp(input, "DEBUG_ON\r")==0)
                {
                    debug = 1;
                    printf("DEBUG IS ON!\r\n");
                } else

                if (strcmp(input, "DEBUG_OFF\r")==0)
                {
                    debug = 0;
                    printf("DEBUG IS OFF!\r\n");
                }
        
                //input[0]='\x9b'; //setting endmarker on beginning of array
                input[0] = '\0'; //setting end of array on the beginning
            
        } else {
            speak = 1;

            j=1;
            while((input[j++]=UART1_Read())!=13 && j<254);
            printf("\n");
            
        }
        
        
        if(speak && !sing){     //Text- or Phoneticmode
            BUSY = 1; //SAM is busy
            input[j++]=10;
            input[j]=0;
            
            input[strlen(input)-2]='\x9b'; //setting array endmark on the position of CR
                                           //(overwrite CR))
            input[strlen(input)-1]= '\0';  //seting end of string vor DEBUG_ON
 
            if (phonetic){
                //input[strlen(input)-2]='\x9b'; //setting array endmark on the position of CR
                //input[strlen(input)-1]= '\0';
                //printf(input);
                SpeakPhoneme(input);
                
            }else speakText(input);
            BUSY = 0; //ready to receive input
        }
        
        
        if(speak && sing){      //Singmode
            BUSY = 1; //SAM is busy
            input[j++]=10;
            input[j]=0;
            
            input[strlen(input)-2]='\x9b'; //setting array endmark on the position of CR
                                           //(overwrite CR))
            input[strlen(input)-1]= '\0';  //seting end of string vor DEBUG_ON
            
            unsigned char singpitch = atoi(input); //Extract SINGPITCH from string
            
            if (debug) {
                printf("\r\nSingPitch %d \r\n",singpitch);
            }
            
            
            j=0;
            while(input[j] != ',') {    //find "," in string
                j++;
            }

                //printf("\r\nj = %d \r\n",j);
            
            
            for(i=0; i<strlen(input); i++) { //move string to eliminate numbers and "," in string
                input[i]=input[i+j+1];
                }
            
            if (debug) {
                printf("\r\n");
                printf(input);
                printf("\r\n");
            }
            
            SingString(singpitch, input);
        
            BUSY = 0; //ready to receive input
        }
        
        
        
        
        
      // end speaking
    }
}


//void singing ()
//{
//}
void ShowInstructionsInfo ()
{
    printf("\r\n");
    printf("\r\n");
    printf("************************************\r\n");
    printf("* Welcome to SAM on the PIC32 V1.0 *\r\n");
    printf("************************************\r\n");
    printf("   (not case sensitive)\r\n");
    printf("---\r\n");
    printf("$HELP      : This page!\r\n");
    printf("$TABLE     : Show PHONEME TABLE\r\n");
    printf("$DEFAULT   : Resets all values to their default values\r\n");
    printf("$STATUS    : Displays current settings\r\n");
    printf("---\r\n");
    printf("$SPEED=    : Change the speed (0-255) default=72\r\n");
    printf("$PITCH=    : Change the pitch (0-255) default=64\r\n");
    printf("$MOUTH=    : Change the mouth (0-255) default=128\r\n");
    printf("$THROAT=   : Change the throat(0-255) default=128\r\n");
    printf("---\r\n");
    printf("$TEXT      : Activate Text mode\r\n");
    printf("$PHONEME   : Activate Phoneme mode\r\n");
    printf("$SING      : Activate Sing mode\r\n");
    printf("           ( pitch,phonemes - wrong input forces reset!!!\r\n");
    printf("             Example: 96,sehehehehehehehehehey )\r\n");
    printf("---\r\n");
    printf("$DEMO1     : Speak DEMO1\r\n");
    printf("$DEMO2     : Speak DEMO2\r\n");
    printf("$DEMO3     : Sing  DEMO3\r\n");
    printf("---\r\n");
    printf("$DEBUG_ON  : Show generated phonemes");
    printf(" (...and leftover garbage from the array ;-)\r\n");
    printf("$DEBUG_OFF : Hide generated phonemes\r\n");
    printf("---\r\n");
    printf("\r\n");
}

void ShowTABLE ()
{
    printf("\r\n");
    printf("---\r\n");
    printf("\r\n");
    printf("     VOWELS                            VOICED CONSONANTS\r\n");
    printf("IY           f(ee)t                    R        red\r\n");
    printf("IH           p(i)n                     L        allow\r\n");
    printf("EH           beg                       W        away\r\n");
    printf("AE           Sam                       W        whale\r\n");
    printf("AA           pot                       Y        you\r\n");
    printf("AH           b(u)dget                  M        Sam\r\n");
    printf("AO           t(al)k                    N        man\r\n");
    printf("OH           cone                      NX       so(ng)\r\n");
    printf("UH           book                      B        bad\r\n");
    printf("UX           l(oo)t                    D        dog\r\n");
    printf("ER           bird                      G        again\r\n");
    printf("AX           gall(o)n                  J        judge\r\n");
    printf("IX           dig(i)t                   Z        zoo\r\n");
    printf("                                       ZH       plea(s)ure\r\n");
    printf("   DIPHTHONGS                          V        seven\r\n");
    printf("EY           m(a)de                    DH       (th)en\r\n");
    printf("AY           h(igh)\r\n");
    printf("OY           boy\r\n");
    printf("AW           h(ow)                     UNVOICED CONSONANTS\r\n");
    printf("OW           slow                      S         Sam\r\n");
    printf("UW           crew                      Sh        fish\r\n");
    printf("                                       F         fish\r\n");
    printf("                                       TH        thin\r\n");
    printf(" SPECIAL PHONEMES                      P         poke\r\n");
    printf("UL           sett(le) (=AXL)           T         talk\r\r\n");
    printf("UM           astron(omy) (=AXM)        K         cake\r\n");
    printf("UN           functi(on) (=AXN)         CH        speech\r\n");
    printf("Q            kitt-en (glottal stop)    /H        a(h)ead\r\n");    
    }

void SpeakDemoText(char* str)
{
	BUSY = 1; //SAM is busy
    printf(str);
	speakText(str);
    delay_ms(200);
    BUSY = 0; //ready to receive input

}

void DemoSpeaking1()
{
	printf("\n\rSAM Speaking Demo 1 in progress...\n\r");
	
	SpeakDemoText("Hello everyone. My name is SAM, short for Software Automatic Mouth.");
	SpeakDemoText("I am ported from SAM on the P C to a P I C 3 2 using P W M for audio.");
	SpeakDemoText("I am so small that I can fit into embedded computers such as P I C and A V R.");
	SpeakDemoText("Honestly, do my voice sound better than the T T S 2 5 6 ?");
	SpeakDemoText("Even though obviously I still sound like a robot to you.");
	SpeakDemoText("Thank you for listening to me. Have a nice day. Good bye!");

	printf("\n\rSAM Speaking Demo 1 done...\n\r");
}

void DemoSpeaking2()
{
	printf("\n\rSAM Speaking Demo 2 in progress...\n\r");
	
	SpeakDemoText("I can do many things, for example, counting numbers:");
	SpeakDemoText("0 1 2 3 4 5 6 7 8 9");
	SpeakDemoText("And spelling letters, A B C D E F G H I J");
	SpeakDemoText("K L M N O P Q R S T U V W X Y Z");
	SpeakDemoText("Although, due to limited memory, some words may not be pronounced correctly.");
	SpeakDemoText("Such as: Windows, Nineteen and Recipe.");
	SpeakDemoText("That should be: Weendows, Nine teen, Re see pee.");
	SpeakDemoText("But I am still good enough to make myself understood. Or at least I hope so");
	SpeakDemoText("Thank you and good bye!");

	printf("\n\rSAM Speaking Demo 2 done...\n\r");
}

void DemoSinging()
{
    
	printf("\n\rSAM Singing Demo in progress...\n\r");

	SpeakDemoText("I will sing the National Anthem of the United States of America");
	SpeakDemoText("The Star Spangled Banner");
    BUSY = 1; //SAM is busy
	SingString(64, "ohohoh");
	SingString(76, "ohohoh");
	SingString(96, "sehehehehehehehehehey");
	SingString(76, "kaeaeaeaeaeaeaeaeaen");
	SingString(64, "yuxuxuxuxuxuxw");
	SingString(48, "siyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiy");
	SingString(38, "baaaaay");
	SingString(42, "dhaaaxaxaxax");
	SingString(48, "daoaoaoaoaoaoaonz");
	SingString(76, "ererererererer");
	SingString(68, "liyiyiyiyiyiyiyiyiy");
	SingString(64, "laaaaaaaaaaaaaaaaaaaaaaaaayt");
	SingString(64, "whahahaht");
	SingString(64, "sohohuw");
	SingString(38, "praaaaaaaaaaaaaaaauwd");
	SingString(42, "liyiyiy");
	SingString(48, "wiyiyiyiyiyiyiyiyiy");
	SingString(51, "/heheheheheheheheheheheheheheheheheheyld");
	SingString(56, "aeaeaeaet");
	SingString(51, "dhaaaxaxaxax");
	SingString(48, "twaaaaaaaaaaaaaaiy");
	SingString(48, "laaaaaaaaaaaaiyts");
	SingString(64, "laeaeaeaeaeaeaeaeaest");
	SingString(76, "gliyiyiyiyiyiyiyiyiym");
	SingString(96, "mihihihihihihihihihihnx");
	SingString(64, "/huxuxuxuxuxuxuxuxuxuxwz");
	SingString(76, "braoaoaod");
	SingString(96, "straaaaaaaaaaaaiyps");
	SingString(76, "aeaeaeaeaeaeaeaeaeaend");
	SingString(64, "braaaaaaaaaaaaaaiyt");
	SingString(48, "staaaaaaaaaaaaaaaaaaaaaaaaaarz");
	SingString(38, "thruxuxw");
	SingString(42, "dhaaaxaxaxax");
	SingString(48, "pehehehehehehehehehehr");
	SingString(76, "rixixixixixixixixixixixixl");
	SingString(68, "lahahahahahahahahahs");
	SingString(64, "faaaaaaaaaaaaaaaaaaaaaaaaaaiyt");
	SingString(64, "ohohohr");
	SingString(64, "dhaaaxaxaxax");
	SingString(38, "raeaeaeaeaeaeaeaeaeaeaeaem");
	SingString(42, "paaaarts");
	SingString(48, "wiyiyiyiyiyiyiyiyiy");
	SingString(51, "waaaaaaaaaaaaaaaaaaaaaaaaaachd");
	SingString(56, "werer");
	SingString(51, "sohohw");
	SingString(48, "gaeaeaeaeaeaeaeaeaeael");
	SingString(48, "lixixixixixixixixixixixixixnt");
	SingString(64, "liyiyiyiyiyiyiyiyiy");
	SingString(76, "striyiyiyiyiyiyiyiyiym");
	SingString(96, "mihihihihihihihihihnx");
	SingString(38, "aeaeaeaeaend");
	SingString(38, "dhaaaxaxaxax");
	SingString(38, "raaaaaaaaaaaak");
	SingString(36, "kixixixixixixixixixixixts");
	SingString(32, "rehehehehehehehehehd");
	SingString(32, "gleheheheheheheheheheheheheheheherer");
	SingString(36, "dhaaaxaxaxax");
	SingString(38, "baaaamz");
	SingString(42, "bererererererst");
	SingString(38, "tihihihihihihihihihnx");
	SingString(36, "ihihihihihihihihihihn");
	SingString(36, "eheheheheheheheheheheheheheheheheheyr");
	SingString(36, "geheheheyv");
	SingString(38, "pruxuxuxuxuxuxuxuxuxuxuxuxwf");
	SingString(42, "thruxuxw");
	SingString(48, "dhaaaxaxax");
	SingString(51, "naaaaaaaaaaaaaaaaaaaaaaayiyt");
	SingString(56, "dhaeaeaeaet");
	SingString(51, "aaaaaauwr");
	SingString(48, "flaeaeaeaeaeaeaeaeaeg");
	SingString(76, "wahahahahahahahahahz");
	SingString(68, "stihihihihihihihihihl");
	SingString(64, "dhehehehehehehehehehehehehehehehehehehehr");
	SingString(64, "ohohohohohohow");
	SingString(48, "sehehehehehehehehehey");
	SingString(48, "dahahahahahahahahahz");
	SingString(48, "dhaeaeae");
	SingString(51, "aeaeaet");
	SingString(56, "staaaaaaaaaaaar");
	SingString(56, "spehehehehehehehehehiynx");
	SingString(56, "gaxaxaxaxaxaxaxaxaxaxaxaxld");
	SingString(42, "baeaeaeaeaeaeaeaeaen");
	SingString(36, "nerer");
	SingString(38, "ererer");
	SingString(42, "yeheheh");
	SingString(48, "eheheheht");
	SingString(48, "weheheheheheheheheheheh");
	SingString(51, "ehehehehehehehiyiyiyv");
	SingString(64, "ohohohr");
	SingString(64, "dhaaaxaxaxax");
	SingString(48, "laeaeaeaeaeaeaeaeaeaeaeae");
	SingString(42, "aeaeaend");
	SingString(38, "ahahahv");
	SingString(36, "dhaaaxaxaxax");
	SingString(32, "friyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiyiy");
	SingString(48, "aeaeaend");
	SingString(42, "dhaaaxaxaxax");
	SingString(38, "/hohohohohohohohohowm");
	SingString(36, "ahahahahv");
	SingString(42, "dhaaaxaxaxaxaxaxaxaxaxaxaxaxax");
	SingString(48, "brehehehehehehehehehehiyiyiyv");
    BUSY = 0; //ready to receive input
	SpeakDemoText("Thank you for listening to me singing. Good bye!");

	printf("\n\rSAM Singing Demo done...\n\r");
    
}